--[[-------------------------------------------------------------------------
oUF_Lolzen by Lolzen 
Layout for oUF by Haste
---------------------------------------------------------------------------]]

--[[	notes:
*debuffhighlighting
]]--

local textfont = "Fonts\\FRIZQT__.TTF"
local textfont2 = "Interface\\AddOns\\oUF_Lolzen\\fonts\\Roadway.ttf"
local hpheight = 12

local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("(.)", string.upper, 1)

	if(unit == "party" or unit == "partypet") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

local getDifficultyColor = function(level)
	local levelDiff = level - UnitLevel("player")
	local color
	if ( levelDiff >= 5 ) then
		color = { r = 1.00, g = 0.10, b = 0.10 }
	elseif ( levelDiff >= 3 ) then
		color = { r = 1.00, g = 0.50, b = 0.25 }
	elseif ( levelDiff >= -2 ) then
		color = { r = 1.00, g = 1.00, b = 1.00 }
	elseif ( -levelDiff <= GetQuestGreenRange() ) then
		color = { r = 0.25, g = 0.75, b = 0.25 }
	else
		color = { r = 0.50, g = 0.50, b = 0.50 }
	end
	return color
end

local updateLevel = function(self, event, unit)
	if(self.unit ~= unit) then return end
	if(unit == "player") then return nil end
	
	local lvl = self.Lvl
	local level = UnitLevel(unit)
	local classification = UnitClassification(unit)

	if(classification=="worldboss") then
		lvl:SetText("|cffff0000Boss|r")
	elseif(classification=="rareelite") then
		lvl:SetText(level.."+ Rare")
	elseif(classification=="elite") then
		lvl:SetText(level.."+")
	elseif(classification=="rare") then
		lvl:SetText(level.." Rare")
	else
		lvl:SetText(level)
	end

	if (UnitCanAttack("player", unit)) then
		local color = getDifficultyColor(level)
		lvl:SetTextColor(color.r, color.g, color.b)
	else
		lvl:SetTextColor(1, 1, 1)
	end

	if level == -1 then
		lvl:SetText("|cffff0000??|r")
	end
end

local updateGlow = function(self, event, unit)
	if(self.unit ~= unit) then return end
	
	local status = UnitThreatSituation(unit)
	if status and status > 0 then
		self.Glow:Show()
	else
		self.Glow:Hide()
	end
end

local colors = setmetatable({
	power = setmetatable({
	["MANA"] = {48/255, 113/255, 191/255},
--	["RAGE"] = {226/255, 45/255, 75/255},
	["FOCUS"] = {255/255, 178/255, 0},
	["ENERGY"] = {1.00, 1.00, 34/255},
	}, {__index = oUF.colors.power}),
}, {__index = oUF.colors})

local updateName = function(self, event, unit)
	if(self.unit == unit) then
		local r, g, b, t
		if(UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit)) then
			r, g, b = .6, .6, .6
		elseif(unit == 'pet') then
			t = self.colors.happiness[GetPetHappiness()]
		else
			r, g, b = 1, 1, 1
		end

		if(t) then
			r, g, b = t[1], t[2], t[3]
		end

		if(r) then
			self.Name:SetTextColor(r, g, b)
		end
	end
end

local updateRIcon = function(self, event)
	local index = GetRaidTargetIndex(self.unit)
	if(index) then
		self.RIcon:SetText(ICON_LIST[index].."22|t")
	else
		self.RIcon:SetText()
	end
end

local siValue = function(val)
	if(val >= 1e6) then
		return ("%.1f".."m"):format(val / 1e6):gsub('%.', '.')
	elseif(val >= 1e4) then
		return ("%.1f".."k"):format(val / 1e3):gsub('%.', '.')
	else
		return val
	end
end

-- replaces name with castname
PostCastStart = function(self, event, unit, spell, spellrank, castid)
	self.Name:SetText(spell)
	self.ClassIcon:SetTexture(nil) -- We just set the texture to nil, so the ClassIcon does not overlay if a cast is spelled.
	if(unit=="player") then
		self.Name:SetAlpha(1)
	end
end

PostCastStop = function(self, event, unit)
	-- Needed as we use it as a general update function.
	if(unit ~= self.unit) then return end
	self.Name:SetText(UnitName(unit))
	self.ClassIcon:SetTexture[[Interface\WorldStateFrame\Icons-Classes]] -- When the Cast ist stopped, we want to see the ClassIcon again
	if(unit == "player") then
		self.Name:SetAlpha(0)
	end
end

local updateHealth = function(self, event, unit, bar, min, max)
	if(max ~= 0) then
		r, g, b = self.ColorGradient(min/max, unpack(self.colors.smooth))
    end
	
	local cur, maxhp = min, max
	
	if(UnitIsDead(unit)) then
		bar.value:SetText"Dead"
		bar.value:SetTextColor(0.6, 0.6, 0.6)
	elseif(UnitIsGhost(unit)) then
		bar.value:SetText"Ghost"
		bar.value:SetTextColor(0.6, 0.6, 0.6)
	elseif(not UnitIsConnected(unit)) then
		bar.value:SetText"Offline"
		bar.value:SetTextColor(0.6, 0.6, 0.6)
	else
		if(floor(cur/maxhp*100) < 100) then
			bar.value:SetFormattedText('|cff%02x%02x%02x%s|r', r*255, g*255, b*255, siValue(cur))
			bar.value:SetAlpha(0.8)
		else
			bar.value:SetText(siValue(max))
			bar.value:SetAlpha(0.8)
		end
		bar.value:SetTextColor(0/255, 255/255, 0/255)
	end
	
	if(self.unit == unit) then
		local r, g, b, t
		if(UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit)) then
			r, g, b = .6, .6, .6
		elseif(UnitIsPlayer(unit)) then
			local _, class = UnitClass(unit)
			t = self.colors.class[class]
		else
			r, g, b = UnitSelectionColor(unit)
		end

		if(t) then
			r, g, b = t[1], t[2], t[3]
		end

		if(r) then
			bar:SetStatusBarColor(r, g, b)
		end
		
		-- Castbarcolor
		if(self.Castbar) then
			local _, class = UnitClass(unit)
			if(class) then
				self.Castbar:SetStatusBarColor(unpack(oUF.colors.class[class]))
			else
				self.Castbar:SetStatusBarColor(1, 1, 1)
			end
		end
		
		-- update for level
		if(unit == "target") then
			updateLevel(self, event, unit)
		end
	end
	
	updateGlow(self, event, unit) -- always fired when updating the health
	
	self.Name:SetText(UnitName(unit))
end

local updatePower = function(self, event, unit, bar, min, max)
local color = oUF.colors.power[UnitPowerType(unit)]
	if(unit == "targettarget") then
		self.Health:SetHeight(hpheight-4)
	else
		self.Health:SetHeight(hpheight)
	end
	self.Power.bg:SetTexture(0, 0, 0, 0)
	if(min == 0) then
		bar.value:SetText()
	elseif(UnitIsDead(unit) or UnitIsGhost(unit)) then
		bar:SetValue(0)
	elseif(not UnitIsConnected(unit)) then
		bar.value:SetText()
	else
		bar.value:SetFormattedText("%s", siValue(min))
		if color then
			bar.value:SetTextColor(color[1], color[2], color[3])
		else
			if(unit == "targettarget") then
				self.Health:SetHeight(hpheight-4)
			elseif(self:GetParent():GetName():match"oUF_Raid") then
				self.Health:SetHeight(hpheight)
			else
				self.Health:SetHeight(hpheight-2)
			end
			self.Power.bg:SetTexture(0, 0, 0, 0.6)
		    bar.value:SetText(siValue(min))
			bar.value:SetTextColor(0.2, 0.66, 0.93, 0.8)
		end
	end
	self.Power.value:SetAlpha(0.6)
	updateGlow(self, event, unit) -- alway fired when updating power
end

local PostCreateAuraIcon = function(self, button)
	local count = button.count
	count:ClearAllPoints()
	count:SetPoint("BOTTOM")

	button.icon:SetTexCoord(.07, .93, .07, .93)
	button.icon:SetPoint("TOPLEFT", button, "TOPLEFT", 1, -1)
	button.icon:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -1, 1)
	
	button.overlay:SetTexture("Interface\\AddOns\\oUF_Lolzen\\textures\\auras")
	button.overlay:SetTexCoord(.03, .97, .03, .97)
	button.overlay.Hide = function(self)
		self:SetVertexColor(0, 0, 0)
	end
end

local PostUpdateAuraIcon
do
	local playerUnits = {
		player = true,
		pet = true,
		vehicle = true,
	}

	PostUpdateAuraIcon = function(self, icons, unit, icon, index, offset, filter, isDebuff)
		local texture = icon.icon
		if(isDebuff) then
			if(playerUnits[icon.owner]) then
				texture:SetDesaturated(false)
			else
				texture:SetDesaturated(true)
			end
		end
	end
end

local func = function(self, unit)
	self.colors = colors
	self.menu = menu

	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)

	self:RegisterForClicks"anyup"
	self:SetAttribute("*type2", "menu")
	
	local Border = CreateFrame("Frame", nil, self)
	Border:SetBackdrop({
		edgeFile = "Interface\\AddOns\\oUF_Lolzen\\textures\\border", edgeSize = 12,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	})
	if(unit == "targettarget") then
		Border:SetPoint("TOPLEFT", self, -3, 3)
		Border:SetPoint("BOTTOMRIGHT", self, 3, -4)
	else
		Border:SetPoint("TOPLEFT", self, -2, 3)
		Border:SetPoint("BOTTOMRIGHT", self, 3, -2)
	end
	Border:SetBackdropBorderColor(0, 0, 0)
	Border:SetFrameLevel(3)
	
	local hp = CreateFrame("StatusBar")
	hp:SetHeight(18)
	hp:SetStatusBarTexture("Interface\\AddOns\\oUF_Lolzen\\textures\\statusbar")
	hp:SetAlpha(0.4)

	hp.frequentUpdates = true

	hp:SetParent(self)
	hp:SetPoint"TOP"
	hp:SetPoint"LEFT"
	hp:SetPoint"RIGHT"

	local hpbg = hp:CreateTexture(nil, "BORDER")
	hpbg:SetAllPoints(self)
	hpbg:SetTexture("Interface\\AddOns\\oUF_Lolzen\\textures\\statusbar")
	hpbg:SetVertexColor(0.5, 0.5, 0.5)
	hpbg:SetAlpha(0.5)
	
	local hpp = Border:CreateFontString(nil, "OVERLAY")
	if(unit == "player") or (unit == "target") then
		hpp:SetPoint("TOPRIGHT", hp,  -3, 12) 
		hpp:SetFont(textfont, 24)
	elseif(unit == "targettarget") then
		hpp:SetPoint("TOPRIGHT", hp, -3, 11)
		hpp:SetFont(textfont, 18)
	elseif(self:GetParent():GetName():sub(1, 9) == "oUF_Party") then
		hpp:SetPoint("RIGHT", hp, -3, 0)
		hpp:SetFont(textfont, 21)
	elseif(self:GetParent():GetName():match"oUF_Raid") then
		hpp:SetPoint("RIGHT", hp, -3, 0)
		hpp:SetFont(textfont, 13)
	elseif(self:GetParent():GetName():match"oUF_MainTank") then
		hpp:SetPoint("TOPRIGHT", hp, -3, 10)
		hpp:SetFont(textfont, 21)
	elseif(unit == "pet") then
		hpp:SetPoint("RIGHT", hp, -3, 10)
		hpp:SetFont(textfont, 21)
	else
		hpp:SetPoint("TOP", hp, 0, 0)
		hpp:SetFont(textfont, 25)
	end
	hpp:SetShadowOffset(1, -1)
	hpp:SetJustifyH("CENTER")
	
	hp.bg = hpbg
	hp.value = hpp
	self.Health = hp
	self.OverrideUpdateHealth = updateHealth

	local pp = CreateFrame("StatusBar")
	pp:SetHeight(2)
	pp:SetStatusBarTexture("Interface\\AddOns\\oUF_Lolzen\\textures\\power")
	pp:SetAlpha(0.4)

	local ppbg = hp:CreateTexture(nil, "BORDER")
	ppbg:SetAllPoints(pp)
	
	pp.frequentUpdates = true
	pp.colorReaction = true
	pp.colorHappiness = true
	pp.colorPower = true
	pp.colorTapping = true
	pp.colorDisconnected = true
	
	pp:SetParent(self)
	pp:SetPoint"LEFT"
	pp:SetPoint"RIGHT"
	pp:SetPoint("TOP", hp, "BOTTOM")
	
	local ppp = Border:CreateFontString(nil, "OVERLAY")
	ppp:SetPoint("BOTTOMRIGHT", hpp, "BOTTOMLEFT", -2, 2)
	ppp:SetFont(textfont, 10)
	ppp:SetShadowOffset(1, -1)

	pp.bg = ppbg
	pp.value = ppp
	self.Power = pp
	self.PostUpdatePower = updatePower
	
	-- FakeFrame for a solid cb-icon
	local ff = CreateFrame("StatusBar")
	ff:SetHeight(0.0000001)
	ff:SetParent(self)
	ff:SetPoint"TOP"
	ff:SetPoint"LEFT"
	ff:SetPoint"RIGHT"

	if(unit == "player" or unit == "target") then
		local cb = CreateFrame("StatusBar")
		cb:SetStatusBarTexture("Interface\\AddOns\\oUF_Lolzen\\textures\\statusbar")
		--cb:SetStatusBarColor(0, 0.85, 1, 1)
		cb:SetParent(ff)
		--cb:SetAllPoints(hp)
		cb:SetAllPoints(self)
		--cb:SetToplevel(true)
		self.Castbar = cb
		
		local icon = cb:CreateTexture(nil, "OVERLAY")
		icon:SetHeight(31)
		icon:SetWidth(31)
		icon:SetTexCoord(.07, .93, .07, .93)
		icon:SetPoint("RIGHT", hp, "LEFT", -4, 8)
		self.Castbar.Icon = icon
		
		local zeit = cb:CreateFontString(nil, "OVERLAY")
		zeit:SetPoint("TOPLEFT", icon, "TOPRIGHT", 8, 2)
		zeit:SetFont(textfont, 12 ,"OUTLINE")
		zeit:SetTextColor(1, 1, 1)
		self.Castbar.Time = zeit
	end
	
	local leader = Border:CreateTexture(nil, "OVERLAY")
	leader:SetHeight(16)
	leader:SetWidth(16)
	if(self:GetParent():GetName():sub(1, 9) == "oUF_Party")then
		leader:SetPoint("TOPRIGHT", hp, "TOPRIGHT", 0, 12)
	elseif(self:GetParent():GetName():match"oUF_Raid") then
		leader:SetPoint("TOPRIGHT", hp, "TOPRIGHT", 0, 12)
	elseif(unit == "player") then
		leader:SetPoint("TOPLEFT", hp, "TOPLEFT", 0, 12)
	else
		leader:Hide()
	end
	self.Leader = leader
--[[
	local runes = CreateFrame("Frame", nil, self)
	runes:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -4)
	runes:SetHeight(4)
	runes:SetWidth(220)
	runes.anchor = "TOPLEFT"
	runes.growth = "RIGHT"
	runes.height = 4
	runes.width = 36
	runes.space = 0.8
	runes.order = {1, 2, 3, 4, 5, 6}
	
	for i = 1, 6 do
		local bar = CreateFrame("Statusbar", nil, runes)
		bar:SetStatusBarTexture("Interface\\AddOns\\oUF_Lolzen\\textures\\statusbar")
		bar.bg = bar:CreateTexture(nil, "BACKGROUND")
		bar.bg:SetTexture("Interface\\AddOns\\oUF_Lolzen\\textures\\statusbar")
		bar.bg:SetAllPoints(bar)
		bar.bg:SetAlpha(0.3)
		bar.bg:SetVertexColor(0, 0, 0)
		bar.border = CreateFrame("Frame", nil, self)
		bar.border:SetBackdrop({
			edgeFile = "Interface\\AddOns\\oUF_Lolzen\\textures\\border", edgeSize = 2,
			insets = {left = 2, right = 2, top = 2, bottom = 2},
		})
		bar.border:SetPoint("TOPLEFT", bar, -2, 2)
		bar.border:SetPoint("BOTTOMRIGHT", bar, 2, -2)
		bar.border:SetBackdropBorderColor(0.2, 0.2, 0.2)
		bar.border:SetFrameLevel(3)

		runes[i] = bar
	end
	self.Runes = runes
	]]
	local ricon = Border:CreateFontString(nil, "OVERLAY")
	if(self:GetParent():GetName():sub(1, 9) == "oUF_Party") then
		ricon:SetPoint("TOPRIGHT", hp, "TOPRIGHT", -15, 12)
	elseif(self:GetParent():GetName():match"oUF_Raid") then
		ricon:SetPoint("TOPRIGHT", hp, "TOPRIGHT", -15, 12)
	elseif(self:GetParent():GetName():match"oUF_MainTank") then
		ricon:SetPoint("LEFT", hp, "RIGHT", 3, 0)
	else
		ricon:SetPoint("CENTER", 0, 10)
	end
	ricon:SetJustifyH"LEFT"
	ricon:SetFontObject(GameFontNormalSmall)
	ricon:SetTextColor(1, 1, 1)
	self.RIcon = ricon
	self:RegisterEvent("RAID_TARGET_UPDATE", updateRIcon)
	table.insert(self.__elements, updateRIcon)
	
	local lvl = Border:CreateFontString(nil, "OVERLAY")
	lvl:SetPoint("LEFT", hp, "LEFT", 2, 5) 
	lvl:SetFont(textfont, 16, "THINOUTLINE")
	self.Lvl = lvl
	
	local name = Border:CreateFontString(nil, "OVERLAY")
	if(unit == "target") then
		name:SetPoint("LEFT", lvl, "RIGHT", 3, 0)
	else
		name:SetPoint("LEFT", self, "LEFT", 2, 5)
	end
	name:SetJustifyH("LEFT")
	if(self:GetParent():GetName():match"oUF_Raid") then
		name:SetFont(textfont2, 13, "THINOUTLINE")
	else
		name:SetFont(textfont2, 16, "THINOUTLINE")
	end
	name:SetTextColor(1, 1, 1, 1)
	if(unit == "targettarget") then
		name:SetPoint("LEFT", self, "LEFT", 2, 4)
		name:SetFont(textfont2, 14, "THINOUTLINE")
	end

	self.Name = name

	if(unit == "target") then
		local cpoints = {}
			for i = 1, 5 do
				cpoints[i] = pp:CreateFontString(nil, "OVERLAY")
				cpoints[i]:SetTextColor(1, 1, 1)
				cpoints[i]:SetFont(textfont2, 20, "THINOUTLINE")
				cpoints[1]:SetText("C")
					if i==2 then
						cpoints[2]:SetText("o")
					end
					if i==3 then
						cpoints[3]:SetText("m")
					end
					if i==4 then
						cpoints[4]:SetText("b")
					end
					if i==5 then
						cpoints[5]:SetText("o")
					end
					if i==1 then
						cpoints[i]:SetPoint("LEFT", hp, "RIGHT")
					else
						cpoints[i]:SetPoint("LEFT", cpoints[i-1], "RIGHT")
					end
			end
		self.CPoints = cpoints
		self.CPoints.unit = 'player'
	end
	
	local glow = CreateFrame("Frame", nil, self)
	glow:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	glow:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
	glow:SetBackdrop({
		edgeFile ="Interface\\AddOns\\oUF_Lolzen\\textures\\glow", edgeSize = 5,
		insets = {left = 4, right = 4, top = 4, bottom = 4}
	})
	glow:SetPoint("TOPLEFT", self, -5, 5)
	glow:SetPoint("BOTTOMRIGHT", self, 5, -5)
	glow:SetBackdropBorderColor(6, 0, 0)
	glow:Hide()
	self.Glow = glow

	if(not unit) then
		local auras = CreateFrame("Frame", nil, self)
		auras:SetHeight(hp:GetHeight() + pp:GetHeight())
		auras:SetWidth(9*18)
		auras:SetPoint("LEFT", self, "RIGHT")

		auras.showDebuffType = true
		auras.size = 18
		auras.gap = true
		auras.numBuffs = 0
		auras.numDebuffs = 0
		self.Auras = auras
	end

	if(unit == "target") then
		local buffs = CreateFrame("Frame", nil, self)
		buffs:SetHeight(18)
		buffs:SetWidth(12*18)
		buffs.initialAnchor = "BOTTOMRIGHT"
		buffs.num = 12
		buffs["growth-x"] = "RIGHT"
		buffs:SetPoint("TOPRIGHT", self, "BOTTOMLEFT", 17, -5)
		buffs.size = 18
		self.Buffs = buffs	
	end

	if(unit and not (unit == "targettarget" or unit == "player")) then
		local debuffs = CreateFrame("Frame", nil, self)
		debuffs:SetHeight(18)
		debuffs:SetWidth(12*20)
		debuffs:SetPoint("TOPRIGHT", self, "BOTTOMLEFT", 17, -30)

		debuffs.showDebuffType = true
		debuffs.size = 18
		debuffs.initialAnchor = "BOTTOMRIGHT"
		debuffs.num = 12
		self.Debuffs = debuffs
	end
	
	if(unit == 'pet') then
		self:RegisterEvent("UNIT_HAPPINESS", updateName)
	end
	
	if(unit == "targettarget") then
		ppp:Hide()
	end

	if(self:GetParent():GetName():sub(1, 9) == "oUF_Party") then
		ppp:Hide()
	end
	
	if(self:GetParent():GetName():match"oUF_Raid") then
		pp:Hide()
		ppp:Hide()
	end
	
	if(self:GetParent():GetName():match"oUF_MainTank") then
		pp:Hide()
		ppp:Hide()
		ppbg:Hide()
	end
	
	if(unit == "pet") then
		ppp:Hide()
	end

	if(not unit) then
		self.Range = true
		self.inRangeAlpha = 1
		self.outsideRangeAlpha = .5
	end
	
	--[[Plugin section]]--
	-- oUF_ClassIcons
	-- class icons
	local class = Border:CreateTexture(nil, "OVERLAY")
	class:SetHeight(32)
	class:SetWidth(32)
	if(unit=="player") or (unit=="target") then
		class:SetPoint("RIGHT", hp, "LEFT", -4, 8)
	end
	self.ClassIcon = class

	if IsAddOnLoaded("oUF_Classicons") then
		local cbg = CreateFrame("Frame", nil, self)
		cbg:SetBackdrop({
			bgFile = "Interface\\AddOns\\oUF_Lolzen\\textures\\statusbar", tile = true, tileSize = 12,
			edgeFile = "Interface\\AddOns\\oUF_Lolzen\\textures\\border", edgeSize = 12,
			insets = {left = 2, right = 2, top = 2, bottom = 2},
		})
		cbg:SetPoint("TOPLEFT", class, -2, 2)
		cbg:SetPoint("BOTTOMRIGHT", class, 2, -2)
		cbg:SetBackdropBorderColor(0, 0, 0)
		cbg:SetBackdropColor(0,0,0,0.5)
		cbg:SetFrameLevel(0)
	end
	
	-- oUF_Smooth
	hp.Smooth = true
	pp.Smooth = true
	
	--[[Plugin section end]]--

	if(unit == "player" or unit == "target" or unit == "pet") then
		self:SetAttribute('initial-height', hpheight)
		self:SetAttribute('initial-width', 220)
	elseif(self:GetParent():GetName():sub(1, 9) == "oUF_Party") then
		self:SetAttribute('initial-height', hpheight)
		self:SetAttribute('initial-width', 110)
	elseif(self:GetParent():GetName():match"oUF_Raid") then
		self:SetAttribute('initial-height', hpheight)
		self:SetAttribute('initial-width', 70)
	elseif(self:GetParent():GetName():match"oUF_Maintank") then
		self:SetAttribute('initial-height', hpheight)
		self:SetAttribute('initial-width', 90)
	else
		self:SetAttribute('initial-height', hpheight-4)
		self:SetAttribute('initial-width', 120)
    end
	
	-- We inject our fake name element early in the cycle, in-case there is a
	-- spell cast in progress on the unit we target.
	self:RegisterEvent('UNIT_NAME_UPDATE', PostCastStop)
	table.insert(self.__elements, 2, PostCastStop)
	
	self.PostChannelStart = PostCastStart
	self.PostCastStart = PostCastStart

	self.PostCastStop = PostCastStop
	self.PostChannelStop = PostCastStop

	self.PostCreateAuraIcon = PostCreateAuraIcon
	self.PostUpdateAuraIcon = PostUpdateAuraIcon

	return self
end

oUF:RegisterStyle("Lolzen", func)

oUF:SetActiveStyle"Lolzen"

local focus = oUF:Spawn"focus"
focus:SetPoint("CENTER", 0, -500)
local pet = oUF:Spawn'pet'
pet:SetPoint('CENTER', 0, -50)
local player = oUF:Spawn"player"
player:SetPoint("CENTER", -250, -50)
local target = oUF:Spawn"target"
target:SetPoint("CENTER", 250, -50)
local tot = oUF:Spawn"targettarget"
tot:SetPoint("CENTER", 300, -27)
local party = oUF:Spawn("header", "oUF_Party")
party:SetPoint("TOPLEFT", 30, -30)
party:SetManyAttributes("showParty", true, "yOffset", -15)
party:Show()
local raid = {}
for i = 1, 8 do
	local raidgroup = oUF:Spawn('header', 'oUF_Raid'..i)
	raidgroup:SetManyAttributes('groupFilter', tostring(i), 'showRaid', true, 'yOffSet', -10, "point", "TOP")
	table.insert(raid, raidgroup)
	if(i==1) then
		raidgroup:SetPoint("TOPLEFT", 30, -30)
	else
		raidgroup:SetPoint('TOPLEFT', raid[i-1], 'TOPRIGHT', 10, 0)
	end
end

local tank = oUF:Spawn('header', 'oUF_MainTank')
tank:SetManyAttributes('showRaid', true, 'groupFilter', 'MAINTANK', 'yOffset', -15)
tank:SetPoint('TOP', UIParent, 0, -30)
tank:Show()

local partyToggle = CreateFrame('Frame')
partyToggle:RegisterEvent('PLAYER_LOGIN')
partyToggle:RegisterEvent('RAID_ROSTER_UPDATE')
partyToggle:RegisterEvent('PARTY_LEADER_CHANGED')
partyToggle:RegisterEvent('PARTY_MEMBERS_CHANGED')
partyToggle:SetScript('OnEvent', function(self)
	if InCombatLockdown() then
		self:RegisterEvent('PLAYER_REGEN_ENABLED')
	else
		self:UnregisterEvent('PLAYER_REGEN_ENABLED')
		local numraid = GetNumRaidMembers()
		if numraid > 0 and (numraid > 5 or numraid ~= GetNumPartyMembers() + 1) then
			party:Hide()
			for i,v in ipairs(raid) do v:Show() end
		else
			party:Show()
			for i,v in ipairs(raid) do v:Hide() end
		end
	end
end)
